@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Authors: Glynn Foster <glynn.foster@sun.com>
  *          Anders Carlsson <andersca@gnu.org>
@@ -90,6 +90,7 @@ static const gchar *author_credits[] = {
   "Hidetoshi Tajima  <hidetoshi tajima sun com>",
   "Tom Tromey  <tromey redhat com>",
   "Yann  <bloch iie cnam fr>",
+  "Norman Rasmussen <normanr gmail com>",
   "",
   "And all the translators that rock my world",
   "==========================================",
@@ -168,13 +169,6 @@ static const gchar *author_credits[] = {
   "Daniel Yacob  <locales geez org>",
   "Funda Wang <fundawang linux net cn>",
   "Alexander Winston  <alexander winston comcast net>",
-  "Adam Weinberger <adamw gnome org>",
-  "Josep Puigdemont <josep imatge-sintetica com>",
-  "Adi Attar <aattar cvs gnome org>",
-  "Steve Murphy <murf e-tools com>",
-  "Abduxukur Abdurixit <abduxukur.abdurixit t-systems ch>",
-  "Ignacio Casal Quinteiro <nacho.resa gmail com>",
-  "Vladimir Petkov <vpetkov i-space org>",
   NULL
 };
 
