@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Authors: Glynn Foster <glynn.foster@sun.com>
  *          Jonathan Blanford <jrb@redhat.com>
@@ -30,14 +30,20 @@
 #include "util.h"
 
 #define MAX_ELEMENTS_BEFORE_SCROLLING 5
+#define PRINT_HIDE_COLUMN_SEPARATOR ","
 
 static GladeXML *glade_dialog;
 static GSList *selected;
 static gchar *separator;
 static gboolean print_all_columns = FALSE;
-static gint print_column_n = 1;
+static gint *print_columns = NULL;
+static gint *hide_columns = NULL;
 
+static int *zenity_tree_extract_column_indexes (char *indexes, gint n_columns);
+static gboolean zenity_tree_column_is_hidden (gint column_index);
 static void zenity_tree_dialog_response (GtkWidget *widget, int response, gpointer data);
+static void zenity_tree_row_activated (GtkTreeView *tree_view, GtkTreePath *tree_path, 
+                                       GtkTreeViewColumn *tree_col, gpointer data);
 
 static gboolean
 zenity_tree_dialog_untoggle (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
@@ -144,11 +150,11 @@ zenity_tree_handle_stdin (GIOChannel  *channel,
       if (toggles && column_count == 0) {
         if (strcmp (g_strdown (zenity_util_strip_newline (string->str)), "true") == 0)
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, TRUE, -1);
-	else
+        else
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, FALSE, -1);
       }
       else {
-        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, zenity_util_strip_newline (string->str), -1);	
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter, column_count, zenity_util_strip_newline (string->str), -1);        
       }
 
     if (editable) {
@@ -218,15 +224,15 @@ zenity_tree_fill_entries (GtkTreeView  *tree_view,
     gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 
     for (j = 0; j < n_columns; j++) {
-	
+        
       if (toggles && j == 0) {
         if (strcmp (g_strdown ((gchar *) args[i+j]), "true") == 0)
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, j, TRUE, -1);
-	else 
+        else 
           gtk_list_store_set (GTK_LIST_STORE (model), &iter, j, FALSE, -1);
       }
       else
-        gtk_list_store_set (GTK_LIST_STORE (model), &iter, j, args[i+j], -1);	
+        gtk_list_store_set (GTK_LIST_STORE (model), &iter, j, args[i+j], -1);        
     }
 
     if (editable)
@@ -296,9 +302,17 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   if (tree_data->print_column) {
     if (strcmp (g_strdown (tree_data->print_column), "all") == 0)
       print_all_columns = TRUE;
-    else
-      print_column_n = atoi (tree_data->print_column);
+    else 
+      print_columns = zenity_tree_extract_column_indexes (tree_data->print_column, n_columns);
   }
+  else { 
+    print_columns = g_new (gint, 2);
+    print_columns[0] = 1;
+    print_columns[1] = 0;
+  }
+
+  if (tree_data->hide_column) 
+    hide_columns = zenity_tree_extract_column_indexes (tree_data->hide_column, n_columns);
 
   if (n_columns == 0) {
     g_printerr (_("No column titles specified for List dialog.\n")); 
@@ -306,6 +320,12 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
     return;
   }
 
+  if (tree_data->checkbox && tree_data->radiobox) {
+    g_printerr (_("You should use only one List dialog type.\n")); 
+    data->exit_code = zenity_util_return_exit_code (ZENITY_ERROR);
+    return;
+  }
+
   glade_xml_signal_autoconnect (glade_dialog);
 
   dialog = glade_xml_get_widget (glade_dialog, "zenity_tree_dialog");
@@ -319,7 +339,7 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
   text = glade_xml_get_widget (glade_dialog, "zenity_tree_text");
                                                                                 
   if (tree_data->dialog_text)
-  gtk_label_set_text (GTK_LABEL (text), tree_data->dialog_text);
+    gtk_label_set_text (GTK_LABEL (text), tree_data->dialog_text);
 
   zenity_util_set_window_icon (dialog, data->window_icon, ZENITY_IMAGE_FULLPATH ("zenity-list.png"));
 
@@ -328,6 +348,10 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
 
   tree_view = glade_xml_get_widget (glade_dialog, "zenity_tree_view");
 
+  if (!(tree_data->radiobox || tree_data->checkbox)) 
+    g_signal_connect (G_OBJECT (tree_view), "row-activated", 
+                      G_CALLBACK (zenity_tree_row_activated), data);
+ 
   /* Create an empty list store */
   model = g_object_new (GTK_TYPE_LIST_STORE, NULL);
 
@@ -354,8 +378,17 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
 
   gtk_tree_view_set_model (GTK_TREE_VIEW (tree_view), GTK_TREE_MODEL (model));
 
-  gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view)),
-                               GTK_SELECTION_MULTIPLE);
+  if (!(tree_data->radiobox || tree_data->checkbox)) {
+    if (tree_data->multi)
+      gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view)),
+                                   GTK_SELECTION_MULTIPLE);
+    else
+      gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view)),
+                                   GTK_SELECTION_SINGLE);
+  } 
+  else  
+    gtk_tree_selection_set_mode (gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view)),
+                                   GTK_SELECTION_NONE);
 
   column_index = 0;
 
@@ -365,13 +398,13 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
         GtkCellRenderer *cell_renderer;
     
         cell_renderer = gtk_cell_renderer_toggle_new ();
-				
+                                
         if (tree_data->radiobox) {
           g_object_set (G_OBJECT (cell_renderer), "radio", TRUE, NULL);
           g_object_set_data (G_OBJECT (model), "radio", (gint *) 1);
         }
 
-	g_signal_connect (cell_renderer, "toggled",
+        g_signal_connect (cell_renderer, "toggled",
                           G_CALLBACK (zenity_tree_toggled_callback), model);
 
         column = gtk_tree_view_column_new_with_attributes (tmp->data,
@@ -393,18 +426,20 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
                                                              "text", column_index, 
                                                              "editable", n_columns, 
                                                              NULL);
-          } 
-          else  {
-            column = gtk_tree_view_column_new_with_attributes (tmp->data, 
-                                                               gtk_cell_renderer_text_new (), 
-                                                               "text", column_index, 
-                                                               NULL);
-          }
+        } 
+        else  {
+          column = gtk_tree_view_column_new_with_attributes (tmp->data, 
+                                                             gtk_cell_renderer_text_new (), 
+                                                             "text", column_index, 
+                                                             NULL);
+        }
 
         gtk_tree_view_column_set_sort_column_id (column, column_index);
         gtk_tree_view_column_set_resizable (column, TRUE);
       }
-			
+      if (zenity_tree_column_is_hidden (1))
+        gtk_tree_view_column_set_visible (column, FALSE);
+ 
       first_column = TRUE;
     }
     else {
@@ -431,8 +466,11 @@ zenity_tree (ZenityData *data, ZenityTreeData *tree_data)
 
       gtk_tree_view_column_set_sort_column_id (column, column_index);
       gtk_tree_view_column_set_resizable (column, TRUE);
+
+      if (zenity_tree_column_is_hidden (column_index + 1))
+        gtk_tree_view_column_set_visible (column, FALSE);
     }
-	
+        
     gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), column);
     column_index++;
   }
@@ -477,8 +515,8 @@ zenity_tree_dialog_get_selected (GtkTreeModel *model, GtkTreePath *path_buf, Gtk
     return;
   }
 
-  if (print_column_n > 0 && print_column_n <= n_columns) {
-    gtk_tree_model_get_value  (model, iter, print_column_n - 1, &value);
+  for (i = 0; print_columns[i] != 0; i++) {
+    gtk_tree_model_get_value  (model, iter, print_columns[i] - 1, &value);
 
     selected = g_slist_append (selected, g_strdup (g_value_get_string (&value)));
     g_value_unset (&value);
@@ -489,7 +527,7 @@ static gboolean
 zenity_tree_dialog_toggle_get_selected (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GtkTreeView *tree_view)
 {
   GValue toggle_value = {0, };
-  gint n_columns, i;
+  gint n_columns, print_column, i;
 
   n_columns = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (tree_view), "n_columns"));
 
@@ -509,14 +547,16 @@ zenity_tree_dialog_toggle_get_selected (GtkTreeModel *model, GtkTreePath *path,
       return FALSE;
     }
 
-    if (print_column_n > 0 && print_column_n <= n_columns) {
-      gtk_tree_model_get_value (model, iter, print_column_n, &value);
+    for (i = 0; print_columns[i] != 0; i++) {
+      gtk_tree_model_get_value (model, iter, print_columns[i], &value);
 
       selected = g_slist_append (selected, g_strdup (g_value_get_string (&value)));
       g_value_unset (&value);
     }
   }
+  
   g_value_unset (&toggle_value);
+
   return FALSE;
 }
 
@@ -533,6 +573,8 @@ zenity_tree_dialog_output (void)
       g_print ("%s\n", (gchar *) tmp->data);
   }
 
+  g_free (print_columns);
+  g_free (hide_columns);
   g_free (separator);
   g_slist_foreach (selected, (GFunc) g_free, NULL);
   selected = NULL;
@@ -575,3 +617,64 @@ zenity_tree_dialog_response (GtkWidget *widget, int response, gpointer data)
   }
   gtk_main_quit ();
 }
+
+static void
+zenity_tree_row_activated (GtkTreeView *tree_view, GtkTreePath *tree_path, 
+                           GtkTreeViewColumn *tree_col, gpointer data)
+{
+  ZenityData *zen_data = data;
+  GtkTreeSelection *selection; 
+  GtkTreeModel *model;
+
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+  gtk_tree_selection_selected_foreach (selection, 
+                                       (GtkTreeSelectionForeachFunc) zenity_tree_dialog_get_selected, 
+                                       GTK_TREE_VIEW (tree_view));
+ 
+  zenity_tree_dialog_output ();
+  zen_data->exit_code = zenity_util_return_exit_code (ZENITY_OK);
+  gtk_main_quit ();
+}
+
+static gboolean
+zenity_tree_column_is_hidden (gint column_index)
+{
+  gint i;
+
+  if (hide_columns != NULL)
+    for (i = 0; hide_columns[i] != 0; i++)
+      if (hide_columns[i] == column_index)
+        return TRUE;
+
+  return FALSE;
+}
+
+static gint *
+zenity_tree_extract_column_indexes (char *indexes, int n_columns)
+{
+  char **tmp;  
+  gint *result;
+  gint i, j, index;
+
+  tmp = g_strsplit (indexes, 
+                    PRINT_HIDE_COLUMN_SEPARATOR, 0);
+
+  result = g_new (gint, 1);
+
+  for (j = i = 0; tmp[i] != NULL; i++) {
+    index = atoi (tmp[i]);
+
+    if (index > 0 && index <= n_columns) {
+      result[j] = index;
+      j++;
+      g_renew (gint, result, j + 1);
+    }
+  }
+  result[j] = 0;
+
+  g_strfreev (tmp);
+
+  return result;
+}
