@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Authors: Glynn Foster <glynn.foster@sun.com>
  *          Havoc Pennington <hp@redhat.com>
@@ -146,13 +146,13 @@ zenity_util_fill_file_buffer (GtkTextBuffer *buffer, const gchar *filename)
 GdkPixbuf *
 zenity_util_pixbuf_new_from_file (GtkWidget *widget, gchar *filename)
 {
-    if (!strcmp (g_strdown (filename), "warning"))
+    if (!strcasecmp (filename, "warning"))
       return gtk_widget_render_icon (widget, GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_BUTTON, NULL);
-    if (!strcmp (g_strdown (filename), "info"))
+    if (!strcasecmp (filename, "info"))
       return gtk_widget_render_icon (widget, GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_BUTTON, NULL);
-    if (!strcmp (g_strdown (filename), "question"))
+    if (!strcasecmp (filename, "question"))
       return gtk_widget_render_icon (widget, GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_BUTTON, NULL);
-    if (!strcmp (g_strdown (filename), "error"))
+    if (!strcasecmp (filename, "error"))
       return gtk_widget_render_icon (widget, GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_BUTTON, NULL);
     else
       return gdk_pixbuf_new_from_file (filename, NULL);
@@ -266,14 +266,22 @@ zenity_util_return_exit_code ( ZenityExitCode value )
 #ifdef GDK_WINDOWING_X11
 
 static Window
-transient_get_xterm ()
+transient_get_xterm (void)
 {
   const char *wid_str = g_getenv ("WINDOWID");
   if (wid_str) {
     char *wid_str_end;
     Window wid = strtoul (wid_str, &wid_str_end, 10);
-    if (*wid_str != '\0' && *wid_str_end == '\0' && wid != 0)
+    if (*wid_str != '\0' && *wid_str_end == '\0' && wid != 0) {
+      XWindowAttributes attrs;
+      gdk_error_trap_push ();
+      XGetWindowAttributes (GDK_DISPLAY(), wid, &attrs);
+      gdk_flush();
+      if (gdk_error_trap_pop () != 0) {
+        return None;
+      }
       return wid;
+    }
   }
   return None;
 }
@@ -303,7 +311,7 @@ transient_is_toplevel (Window wid)
  */
 
 static Window
-transient_get_xterm_toplevel ()
+transient_get_xterm_toplevel (void)
 {
   Window xterm = transient_get_xterm ();
   Display *dpy = GDK_DISPLAY ();
